@@ -9,8 +9,8 @@
 #include <QTimer>
 #include <QEventLoop>
 
-OnlineSolver::OnlineSolver(ProcessType type, ExtractorType sexType, SolverType solType, FITSImage::Statistic imagestats,
-                           uint8_t const *imageBuffer, QObject *parent) : ExternalSextractorSolver(type, sexType, solType, imagestats, imageBuffer,
+OnlineSolver::OnlineSolver(ProcessType type, ExtractorType exType, SolverType solType, const FITSImage::Statistic &imagestats,
+                           uint8_t const *imageBuffer, QObject *parent) : ExternalExtractorSolver(type, exType, solType, imagestats, imageBuffer,
                                        parent)
 {
     connect(this, &OnlineSolver::timeToCheckJobs, this, &OnlineSolver::checkJobs);
@@ -35,9 +35,9 @@ void OnlineSolver::execute()
         ycol = strdup("Y"); //This is the column for the y-coordinates, it doesn't accept Y_IMAGE like the other one
         int fail = 0;
         if(m_ExtractorType == EXTRACTOR_INTERNAL)
-            fail = runSEPSextractor();
+            fail = runSEPExtractor();
         else if(m_ExtractorType == EXTRACTOR_EXTERNAL)
-            fail = runExternalSextractor();
+            fail = runExternalExtractor();
         if(fail != 0)
         {
             emit finished(fail);
@@ -49,7 +49,7 @@ void OnlineSolver::execute()
             emit finished(-1);
             return;
         }
-        if((fail = writeSextractorTable()) != 0)
+        if((fail = writeStarExtractorTable()) != 0)
         {
             emit finished(fail);
             return;
@@ -71,6 +71,8 @@ void OnlineSolver::runOnlineSolver()
             QFile(m_LogFileName).remove();
     }
 
+    m_WasAborted = false;
+
     solverTimer.start();
 
     emit startupOnlineSolver(); //Go to FIRST STAGE
@@ -85,13 +87,13 @@ void OnlineSolver::run()
 {
     bool timedOut = false;
 
-    while(!m_HasSolved && !aborted && !timedOut && workflowStage != JOB_PROCESSING_STAGE)
+    while(!m_HasSolved && !m_WasAborted && !timedOut && workflowStage != JOB_PROCESSING_STAGE)
     {
         msleep(200);
         timedOut = solverTimer.elapsed() / 1000.0 > m_ActiveParameters.solverTimeLimit;
     }
 
-    while(!m_HasSolved && !aborted && !timedOut && (workflowStage == JOB_PROCESSING_STAGE || workflowStage == JOB_QUEUE_STAGE))
+    while(!m_HasSolved && !m_WasAborted && !timedOut && (workflowStage == JOB_PROCESSING_STAGE || workflowStage == JOB_QUEUE_STAGE))
     {
         msleep(JOB_RETRY_DURATION);
         if (job_retries++ > JOB_RETRY_ATTEMPTS)
@@ -106,21 +108,21 @@ void OnlineSolver::run()
         }
     }
 
-    if(!m_HasSolved && !aborted && !timedOut)
+    if(!m_HasSolved && !m_WasAborted && !timedOut)
     {
         emit logOutput("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
         emit logOutput("Starting Online Solver with the " + m_ActiveParameters.listName + " profile . . .");
 
     }
 
-    while(!m_HasSolved && !aborted && !timedOut && workflowStage == JOB_MONITORING_STAGE)
+    while(!m_HasSolved && !m_WasAborted && !timedOut && workflowStage == JOB_MONITORING_STAGE)
     {
         msleep(STATUS_CHECK_INTERVAL);
         emit timeToCheckJobs();
         timedOut = solverTimer.elapsed() / 1000.0 > m_ActiveParameters.solverTimeLimit;
     }
 
-    if(aborted)
+    if(m_WasAborted)
         return;
 
     if(timedOut)
@@ -137,7 +139,7 @@ void OnlineSolver::run()
     bool starsAndWCSTimedOut = false;
     double starsAndWCSTimeLimit = 10.0;
 
-    if(!aborted && !m_HasWCS)
+    if(!m_WasAborted && !m_HasWCS)
     {
         emit logOutput("Waiting for Stars and WCS. . .");
         solverTimer.start();  //Restart the timer for the Stars and WCS download
@@ -145,7 +147,7 @@ void OnlineSolver::run()
 
     //This will wait for stars and WCS until the time limit is reached
     //If it does get the file, whether or not it can read it, the stage changes to NO_STAGE and this quits
-    while(!aborted && !starsAndWCSTimedOut && (workflowStage == LOG_LOADING_STAGE || workflowStage == WCS_LOADING_STAGE))
+    while(!m_WasAborted && !starsAndWCSTimedOut && (workflowStage == LOG_LOADING_STAGE || workflowStage == WCS_LOADING_STAGE))
     {
         msleep(STATUS_CHECK_INTERVAL);
         starsAndWCSTimedOut = solverTimer.elapsed() / 1000.0 > starsAndWCSTimeLimit; //Wait 10 seconds for STARS and WCS, NO LONGER!
@@ -167,7 +169,8 @@ void OnlineSolver::abort()
     workflowStage  = NO_STAGE;
     emit logOutput("Online Solver aborted.");
     emit finished(-1);
-    aborted = true;
+    quit();
+    m_WasAborted = true;
 }
 
 //This will start up the first stage, Authentication
@@ -206,11 +209,11 @@ void OnlineSolver::uploadFile()
     if(m_ExtractorType == EXTRACTOR_BUILTIN)
         fitsFile = new QFile(fileToProcess);
     else
-        fitsFile = new QFile(sextractorFilePath);
+        fitsFile = new QFile(starXYLSFilePath);
     bool rc = fitsFile->open(QIODevice::ReadOnly);
     if (rc == false)
     {
-        emit logOutput(QString("Failed to open the file %1: %2").arg( fileToProcess).arg( fitsFile->errorString()));
+        emit logOutput(QString("Failed to open the file %1: %2").arg( fileToProcess, fitsFile->errorString()));
         delete (fitsFile);
         emit finished(-1);
         return;
@@ -419,7 +422,7 @@ void OnlineSolver::onResult(QNetworkReply *reply)
             sessionKey = result["session"].toString();
 
             if(m_SSLogLevel != LOG_OFF)
-                emit logOutput(QString("Authentication to %1 is successful. Session: %2").arg(astrometryAPIURL).arg(sessionKey));
+                emit logOutput(QString("Authentication to %1 is successful. Session: %2").arg(astrometryAPIURL, sessionKey));
 
             uploadFile(); //Go to NEXT STAGE
             break;
@@ -549,7 +552,7 @@ void OnlineSolver::onResult(QNetworkReply *reply)
                 raErr = (search_ra - ra) * 3600;
                 decErr = (search_dec - dec) * 3600;
             }
-            QString par = (parity > 0) ? "neg" : "pos";
+            FITSImage::Parity par = (parity > 0) ? FITSImage::NEGATIVE : FITSImage::POSITIVE;
             m_Solution = {fieldw, fieldh, ra, dec, orientation, pixscale, par, raErr, decErr};
             m_HasSolved = true;
 
