@@ -14,143 +14,209 @@
 #endif
 
 #include "stellarsolver.h"
-#include "sextractorsolver.h"
-#include "externalsextractorsolver.h"
+#include "extractorsolver.h"
+#include "externalextractorsolver.h"
 #include "onlinesolver.h"
 #include <QApplication>
 #include <QSettings>
 
 using namespace SSolver;
 
-StellarSolver::StellarSolver(ProcessType type, const FITSImage::Statistic &imagestats, const uint8_t *imageBuffer,
-                             QObject *parent) : QObject(parent), m_Statistics(imagestats)
+StellarSolver::StellarSolver(QObject *parent) : QObject(parent)
 {
-    qRegisterMetaType<SolverType>("SolverType");
-    qRegisterMetaType<ProcessType>("ProcessType");
-    qRegisterMetaType<ExtractorType>("ExtractorType");
-    m_ProcessType = type;
-    m_ImageBuffer = imageBuffer;
-    m_Subframe = QRect(0, 0, m_Statistics.width, m_Statistics.height);
+    registerMetaTypes();
 }
 
 StellarSolver::StellarSolver(const FITSImage::Statistic &imagestats, uint8_t const *imageBuffer,
-                             QObject *parent) : QObject(parent), m_Statistics(imagestats)
+                             QObject *parent) : QObject(parent)
 {
-    qRegisterMetaType<SolverType>("SolverType");
-    qRegisterMetaType<ProcessType>("ProcessType");
-    qRegisterMetaType<ExtractorType>("ExtractorType");
-    m_ImageBuffer = imageBuffer;
-    m_Subframe = QRect(0, 0, m_Statistics.width, m_Statistics.height);
+    registerMetaTypes();
+    loadNewImageBuffer(imagestats, imageBuffer);
 }
+
+StellarSolver::StellarSolver(ProcessType type, const FITSImage::Statistic &imagestats, const uint8_t *imageBuffer,
+                             QObject *parent) : QObject(parent)
+{
+    registerMetaTypes();
+    m_ProcessType = type;
+    loadNewImageBuffer(imagestats, imageBuffer);
+}
+
 StellarSolver::~StellarSolver()
 {
 
 }
 
-SextractorSolver* StellarSolver::createSextractorSolver()
+void StellarSolver::registerMetaTypes()
 {
-    SextractorSolver *solver;
+    qRegisterMetaType<SolverType>("SolverType");
+    qRegisterMetaType<ProcessType>("ProcessType");
+    qRegisterMetaType<ExtractorType>("ExtractorType");
+}
+
+bool StellarSolver::loadNewImageBuffer(const FITSImage::Statistic &imagestats, uint8_t const *imageBuffer)
+{
+    if(imageBuffer == nullptr)
+        return false;
+    if(isRunning())
+        return false;
+    m_ImageBuffer = imageBuffer;
+    m_Statistics = imagestats;
+    m_Subframe = QRect(0, 0, m_Statistics.width, m_Statistics.height);
+
+    //information that should be reset since it was about the last image
+    m_HasExtracted = false;
+    m_HasSolved = false;
+    m_HasFailed = false;
+    hasWCS = false;
+    m_isRunning = false;
+    m_UsePosition = false;
+    m_SearchRA = HUGE_VAL;
+    m_SearchDE = HUGE_VAL;
+    m_UseScale = false;
+    m_ScaleHigh = 0;
+    m_ScaleLow = 0;
+    qDeleteAll(parallelSolvers);
+    parallelSolvers.clear();
+    m_ExtractorSolver.reset();
+    m_ParallelSolversFinishedCount = 0;
+    background = {};
+    m_ExtractorStars.clear();
+    m_SolverStars.clear();
+    numStars = 0;
+    solution = {};
+    solutionIndexNumber = -1;
+    solutionHealpix = -1;
+
+    return true;
+}
+
+ExtractorSolver* StellarSolver::createExtractorSolver()
+{
+    ExtractorSolver *solver;
 
     if(m_ProcessType == SOLVE && m_SolverType == SOLVER_ONLINEASTROMETRY)
     {
-        OnlineSolver *onlineSolver = new OnlineSolver(m_ProcessType, m_SextractorType, m_SolverType, m_Statistics, m_ImageBuffer,
+        OnlineSolver *onlineSolver = new OnlineSolver(m_ProcessType, m_ExtractorType, m_SolverType, m_Statistics, m_ImageBuffer,
                 this);
         onlineSolver->fileToProcess = m_FileToProcess;
         onlineSolver->astrometryAPIKey = m_AstrometryAPIKey;
         onlineSolver->astrometryAPIURL = m_AstrometryAPIURL;
-        onlineSolver->sextractorBinaryPath = m_SextractorBinaryPath;
+        onlineSolver->externalPaths = m_ExternalPaths;
         solver = onlineSolver;
     }
     else if((m_ProcessType == SOLVE && m_SolverType == SOLVER_STELLARSOLVER) || (m_ProcessType != SOLVE
-            && m_SextractorType != EXTRACTOR_EXTERNAL))
-        solver = new InternalSextractorSolver(m_ProcessType, m_SextractorType, m_SolverType, m_Statistics, m_ImageBuffer, this);
+            && m_ExtractorType != EXTRACTOR_EXTERNAL))
+        solver = new InternalExtractorSolver(m_ProcessType, m_ExtractorType, m_SolverType, m_Statistics, m_ImageBuffer, this);
     else
     {
-        ExternalSextractorSolver *extSolver = new ExternalSextractorSolver(m_ProcessType, m_SextractorType, m_SolverType,
+        ExternalExtractorSolver *extSolver = new ExternalExtractorSolver(m_ProcessType, m_ExtractorType, m_SolverType,
                 m_Statistics, m_ImageBuffer, this);
         extSolver->fileToProcess = m_FileToProcess;
-        extSolver->sextractorBinaryPath = m_SextractorBinaryPath;
-        extSolver->confPath = m_ConfPath;
-        extSolver->solverPath = m_SolverPath;
-        extSolver->astapBinaryPath = m_ASTAPBinaryPath;
-        extSolver->wcsPath = m_WCSPath;
+        extSolver->externalPaths = m_ExternalPaths;
         extSolver->cleanupTemporaryFiles = m_CleanupTemporaryFiles;
         extSolver->autoGenerateAstroConfig = m_AutoGenerateAstroConfig;
-        extSolver->onlySendFITSFiles = m_OnlySendFITSFiles;
         solver = extSolver;
     }
 
     if(useSubframe)
         solver->setUseSubframe(m_Subframe);
+    solver->m_ColorChannel = m_ColorChannel;
     solver->m_LogToFile = m_LogToFile;
     solver->m_LogFileName = m_LogFileName;
     solver->m_AstrometryLogLevel = m_AstrometryLogLevel;
     solver->m_SSLogLevel = m_SSLogLevel;
     solver->m_BasePath = m_BasePath;
     solver->m_ActiveParameters = params;
+    solver->convFilter = convFilter;
     solver->indexFolderPaths = indexFolderPaths;
+    solver->indexFiles = m_IndexFilePaths;
     if(m_UseScale)
         solver->setSearchScale(m_ScaleLow, m_ScaleHigh, m_ScaleUnit);
     if(m_UsePosition)
         solver->setSearchPositionInDegrees(m_SearchRA, m_SearchDE);
     if(m_SSLogLevel != LOG_OFF)
-        connect(solver, &SextractorSolver::logOutput, this, &StellarSolver::logOutput);
+        connect(solver, &ExtractorSolver::logOutput, this, &StellarSolver::logOutput);
 
     return solver;
 }
 
-//Methods to get default file paths
-ExternalProgramPaths StellarSolver::getLinuxDefaultPaths()
+ExternalProgramPaths StellarSolver::getDefaultExternalPaths(ComputerSystemType system)
 {
-    return ExternalSextractorSolver::getLinuxDefaultPaths();
-};
-ExternalProgramPaths StellarSolver::getLinuxInternalPaths()
-{
-    return ExternalSextractorSolver::getLinuxInternalPaths();
-};
-ExternalProgramPaths StellarSolver::getMacHomebrewPaths()
-{
-    return ExternalSextractorSolver::getMacHomebrewPaths();
-};
-ExternalProgramPaths StellarSolver::getMacInternalPaths()
-{
-    return ExternalSextractorSolver::getMacInternalPaths();
-};
-ExternalProgramPaths StellarSolver::getWinANSVRPaths()
-{
-    return ExternalSextractorSolver::getWinANSVRPaths();
-};
-ExternalProgramPaths StellarSolver::getWinCygwinPaths()
-{
-    return ExternalSextractorSolver::getLinuxDefaultPaths();
+    return ExternalExtractorSolver::getDefaultExternalPaths(system);
 };
 
-void StellarSolver::extract(bool calculateHFR, QRect frame)
+QStringList StellarSolver::getIndexFiles(const QStringList &directoryList, int indexToUse, int healpixToUse)
+{
+    QStringList indexFileList;
+    for(int i = 0; i < directoryList.count(); i++)
+    {
+        const QString &currentPath = directoryList[i];
+        QDir dir(currentPath);
+        QStringList dirIndexFiles;
+        if(dir.exists())
+        {
+            if(indexToUse < 0)
+            {
+                // Find all fits files in the folder.
+                dir.setNameFilters(QStringList() << "*.fits" << "*.fit");
+                dirIndexFiles << dir.entryList();
+            }
+            else
+            {
+                QString name1, name2;
+                if (healpixToUse >= 0)
+                {
+                    // Find all the fits files in the folder associated with the index and healpix.
+                    QString hStr = QString("%1").arg(healpixToUse, 2, 10, (QChar) '0');
+                    name1 = "index-" + QString::number(indexToUse) + "-" + hStr + ".fits";
+                    name2 = "index-" + QString::number(indexToUse) + "-" + hStr + ".fit";
+                }
+                else
+                {
+                    // Find all the fits files in the folder associated with the index number.
+                    name1 = "index-" + QString::number(indexToUse) + "*.fits";
+                    name2 = "index-" + QString::number(indexToUse) + "*.fit";
+                }
+                dir.setNameFilters(QStringList() << name1 << name2);
+                dirIndexFiles << dir.entryList();
+            }
+            for(int i = 0; i < dirIndexFiles.count(); i++)
+            {
+               indexFileList.append(dir.absolutePath() + QDir::separator() + dirIndexFiles.at(i));
+            }
+        }
+    }
+    return indexFileList;
+}
+
+bool StellarSolver::extract(bool calculateHFR, QRect frame)
 {
     m_ProcessType = calculateHFR ? EXTRACT_WITH_HFR : EXTRACT;
     useSubframe = !frame.isNull() && frame.isValid();
     if (useSubframe)
         m_Subframe = frame;
+
+    // This loop will wait syncrounously for the finished signal that the star extraction is done
+    QEventLoop loop;
+    connect(this, &StellarSolver::finished, &loop, &QEventLoop::quit);
     start();
-    m_SextractorSolver->wait();
+    loop.exec(QEventLoop::ExcludeUserInputEvents);
+
+    return m_HasExtracted;
 }
 
-//This will allow the solver to gracefully disconnect, abort, finish, and get deleted
-//Right now the internal solvers are all deleted when StellarSolver is deleted
-//I might try experimenting with this.
-void StellarSolver::releaseSextractorSolver(SextractorSolver *solver)
+bool StellarSolver::solve()
 {
-    if(solver != nullptr)
-    {
-        if(solver->isRunning())
-        {
-            connect(solver, &SextractorSolver::finished, solver, &SextractorSolver::deleteLater);
-            solver->disconnect(this);
-            solver->abort();
-        }
-        else
-            solver->deleteLater();
-    }
+    m_ProcessType = SOLVE;
+
+    // This loop will wait syncrounously for the finished signal that the solving is done
+    QEventLoop loop;
+    connect(this, &StellarSolver::finished, &loop, &QEventLoop::quit);
+    start();
+    loop.exec(QEventLoop::ExcludeUserInputEvents);
+
+    return m_HasSolved;
 }
 
 void StellarSolver::start()
@@ -165,7 +231,10 @@ void StellarSolver::start()
         return;
     }
 
-    m_SextractorSolver = createSextractorSolver();
+    //This is necessary before starting up so that the correct convolution filter gets passed to the ExtractorSolver
+    updateConvolutionFilter();
+
+    m_ExtractorSolver.reset(createExtractorSolver());
 
     m_isRunning = true;
     m_HasFailed = false;
@@ -179,22 +248,17 @@ void StellarSolver::start()
         m_SolverStars.clear();
         m_HasSolved = false;
         hasWCS = false;
-        hasWCSCoord = false;
-        wcs_coord = nullptr;
     }
 
-    //    if(m_isBlocking)
-    //        return;
-
     //These are the solvers that support parallelization, ASTAP and the online ones do not
     if(params.multiAlgorithm != NOT_MULTI && m_ProcessType == SOLVE && (m_SolverType == SOLVER_STELLARSOLVER
             || m_SolverType == SOLVER_LOCALASTROMETRY))
     {
         //Note that it is good to do the Star Extraction before parallelization because it doesn't make sense to repeat this step in all the threads, especially since SEP is now also parallelized in StellarSolver.
-        if(m_SextractorType != EXTRACTOR_BUILTIN)
+        if(m_ExtractorType != EXTRACTOR_BUILTIN)
         {
-            m_SextractorSolver->extract();
-            if(m_SextractorSolver->getNumStarsFound() == 0)
+            m_ExtractorSolver->extract();
+            if(m_ExtractorSolver->getNumStarsFound() == 0)
             {
                 emit logOutput("No stars were found, so the image cannot be solved");
                 m_isRunning = false;
@@ -205,42 +269,64 @@ void StellarSolver::start()
             }
         }
         //Note that converting the image to a FITS file if desired, doesn't need to be repeated in all the threads, but also CFITSIO fails when accessed by multiple parallel threads.
-        if(m_SolverType == SOLVER_LOCALASTROMETRY && m_SextractorType == EXTRACTOR_BUILTIN && m_OnlySendFITSFiles)
+        if(m_SolverType == SOLVER_LOCALASTROMETRY && m_ExtractorType == EXTRACTOR_BUILTIN)
         {
-            QPointer<ExternalSextractorSolver> extSolver = (ExternalSextractorSolver*) (SextractorSolver*) m_SextractorSolver;
+            ExternalExtractorSolver *extSolver = static_cast<ExternalExtractorSolver*> (m_ExtractorSolver.data());
             QFileInfo file(extSolver->fileToProcess);
-            if(file.suffix() != "fits" && file.suffix() != "fit")
+            int ret = extSolver->saveAsFITS();
+            if(ret != 0)
             {
-                int ret = extSolver->saveAsFITS();
-                if(ret != 0)
-                {
-                    emit logOutput("Failed to save FITS File.");
-                    return;
-                }
+                emit logOutput("Failed to save FITS File.");
+                return;
             }
         }
+        // There is no reason to generate a bunch of copies of the config file, just one will do for all the parallel threads.
+        if(m_SolverType == SOLVER_LOCALASTROMETRY)
+        {
+            ExternalExtractorSolver *extSolver = static_cast<ExternalExtractorSolver*> (m_ExtractorSolver.data());
+            extSolver->generateAstrometryConfigFile();
+        }
         parallelSolve();
     }
     else if(m_SolverType == SOLVER_ONLINEASTROMETRY)
     {
-        connect(m_SextractorSolver, &SextractorSolver::finished, this, &StellarSolver::processFinished);
-        m_SextractorSolver->execute();
+        OnlineSolver *onSolver = static_cast<OnlineSolver*> (m_ExtractorSolver.data());
+        int ret = onSolver->saveAsFITS();
+        if(ret != 0)
+        {
+            emit logOutput("Failed to save FITS File.");
+            return;
+        }
+        connect(m_ExtractorSolver.data(), &ExtractorSolver::finished, this, &StellarSolver::processFinished);
+        m_ExtractorSolver->execute();
     }
     else
     {
-        connect(m_SextractorSolver, &SextractorSolver::finished, this, &StellarSolver::processFinished);
-        m_SextractorSolver->start();
+        connect(m_ExtractorSolver.data(), &ExtractorSolver::finished, this, &StellarSolver::processFinished);
+        m_ExtractorSolver->start();
     }
 
 }
 
 bool StellarSolver::checkParameters()
 {
-    if(m_SolverType == SOLVER_ASTAP && m_SextractorType != EXTRACTOR_BUILTIN)
+    if(m_ImageBuffer == nullptr)
+    {
+        emit logOutput("The image buffer is not loaded, please load an image before processing it");
+        return false;
+    }
+
+    if(m_ProcessType == SOLVE && m_SolverType == SOLVER_WATNEYASTROMETRY && (m_Statistics.dataType == SEP_TFLOAT || m_Statistics.dataType == SEP_TDOUBLE))
+    {
+        emit logOutput("The Watney Solver cannot solve floating point images.");
+        return false;
+    }
+
+    if(m_SolverType == SOLVER_ASTAP && m_ExtractorType != EXTRACTOR_BUILTIN)
     {
         if(m_SSLogLevel != LOG_OFF)
             emit logOutput("ASTAP no longer supports alternative star extraction methods.  Changing to built-in star extraction.");
-        m_SextractorType = EXTRACTOR_BUILTIN;
+        m_ExtractorType = EXTRACTOR_BUILTIN;
     }
 
     if(params.multiAlgorithm != NOT_MULTI && m_SolverType == SOLVER_ASTAP && m_ProcessType == SOLVE)
@@ -261,11 +347,11 @@ bool StellarSolver::checkParameters()
 
     if(m_ProcessType == SOLVE && m_SolverType != SOLVER_ASTAP)
     {
-        if(m_SolverType == SOLVER_STELLARSOLVER && m_SextractorType != EXTRACTOR_INTERNAL)
+        if(m_SolverType == SOLVER_STELLARSOLVER && m_ExtractorType != EXTRACTOR_INTERNAL)
         {
             if(m_SSLogLevel != LOG_OFF)
-                emit logOutput("StellarSolver only uses the Internal SEP Sextractor since it doesn't save files to disk. Changing to Internal Star Extractor.");
-            m_SextractorType = EXTRACTOR_INTERNAL;
+                emit logOutput("StellarSolver only uses the Internal SEP Star Extractor since it doesn't save files to disk. Changing to Internal Star Extractor.");
+            m_ExtractorType = EXTRACTOR_INTERNAL;
         }
 
         if(params.multiAlgorithm == MULTI_AUTO)
@@ -280,6 +366,12 @@ bool StellarSolver::checkParameters()
                 params.multiAlgorithm = MULTI_SCALES;
         }
 
+        if(m_ProcessType == SOLVE && m_SolverType == SOLVER_WATNEYASTROMETRY && params.keepNum < 300)
+        {
+            emit logOutput("The Watney Solver needs at least 300 stars. Adjusting keepNum to 300");
+            params.keepNum = 300;
+        }
+
         if(params.inParallel)
         {
             if(enoughRAMisAvailableFor(indexFolderPaths))
@@ -299,7 +391,7 @@ bool StellarSolver::checkParameters()
         }
     }
 
-    return true; //For now
+    return true;
 }
 
 //This allows us to start multiple threads to search simulaneously in separate threads/cores
@@ -308,6 +400,7 @@ void StellarSolver::parallelSolve()
 {
     if(params.multiAlgorithm == NOT_MULTI || !(m_SolverType == SOLVER_STELLARSOLVER || m_SolverType == SOLVER_LOCALASTROMETRY))
         return;
+    qDeleteAll(parallelSolvers);
     parallelSolvers.clear();
     m_ParallelSolversFinishedCount = 0;
     int threads = QThread::idealThreadCount();
@@ -335,12 +428,12 @@ void StellarSolver::parallelSolve()
         double scaleConst = (maxScale - minScale) / pow(threads, 2);
         if(m_SSLogLevel != LOG_OFF)
             emit logOutput(QString("Starting %1 threads to solve on multiple scales").arg(threads));
-        for(double thread = 0; thread < threads; thread++)
+        for(int thread = 0; thread < threads; thread++)
         {
             double low = minScale + scaleConst * pow(thread, 2);
             double high = minScale + scaleConst * pow(thread + 1, 2);
-            SextractorSolver *solver = m_SextractorSolver->spawnChildSolver(thread);
-            connect(solver, &SextractorSolver::finished, this, &StellarSolver::finishParallelSolve);
+            ExtractorSolver *solver = m_ExtractorSolver->spawnChildSolver(thread);
+            connect(solver, &ExtractorSolver::finished, this, &StellarSolver::finishParallelSolve);
             solver->setSearchScale(low, high, units);
             parallelSolvers.append(solver);
             if(m_SSLogLevel != LOG_OFF)
@@ -365,8 +458,8 @@ void StellarSolver::parallelSolve()
             emit logOutput(QString("Starting %1 threads to solve on multiple depths").arg(sourceNum / inc));
         for(int i = 1; i < sourceNum; i += inc)
         {
-            SextractorSolver *solver = m_SextractorSolver->spawnChildSolver(i);
-            connect(solver, &SextractorSolver::finished, this, &StellarSolver::finishParallelSolve);
+            ExtractorSolver *solver = m_ExtractorSolver->spawnChildSolver(i);
+            connect(solver, &ExtractorSolver::finished, this, &StellarSolver::finishParallelSolve);
             solver->depthlo = i;
             solver->depthhi = i + inc;
             parallelSolvers.append(solver);
@@ -374,63 +467,58 @@ void StellarSolver::parallelSolve()
                 emit logOutput(QString("Child Solver # %1, Depth Low %2, Depth High %3").arg(parallelSolvers.count()).arg(i).arg(i + inc));
         }
     }
-    for(auto solver : parallelSolvers)
+    for(auto &solver : parallelSolvers)
         solver->start();
 }
 
 bool StellarSolver::parallelSolversAreRunning() const
 {
-    for(auto solver : parallelSolvers)
+    for(const auto &solver : parallelSolvers)
         if(solver->isRunning())
             return true;
     return false;
 }
 void StellarSolver::processFinished(int code)
 {
-    numStars  = m_SextractorSolver->getNumStarsFound();
+    numStars  = m_ExtractorSolver->getNumStarsFound();
     if(code == 0)
     {
-        if(m_ProcessType == SOLVE && m_SextractorSolver->solvingDone())
+        if(m_ProcessType == SOLVE && m_ExtractorSolver->solvingDone())
         {
-            solution = m_SextractorSolver->getSolution();
-            m_SolverStars = m_SextractorSolver->getStarList();
-            if(m_SextractorSolver->hasWCSData())
+            solution = m_ExtractorSolver->getSolution();
+            solutionIndexNumber = m_ExtractorSolver->getSolutionIndexNumber();
+            solutionHealpix = m_ExtractorSolver->getSolutionHealpix();
+            m_SolverStars = m_ExtractorSolver->getStarList();
+            if(m_ExtractorSolver->hasWCSData())
             {
                 hasWCS = true;
-                solverWithWCS = m_SextractorSolver;
-                if(loadWCS)
-                {
-                    solverWithWCS->computingWCS = true;
-                    disconnect(solverWithWCS, &SextractorSolver::finished, this, &StellarSolver::processFinished);
-                    connect(solverWithWCS, &SextractorSolver::finished, this, &StellarSolver::finishWCS);
-                    solverWithWCS->start();
-                }
+                wcsData = m_ExtractorSolver->getWCSData();
+                wcsData = m_ExtractorSolver->getWCSData();
+                if(m_ExtractorStars.count() > 0)
+                    wcsData.appendStarsRAandDEC(m_ExtractorStars);
             }
             m_HasSolved = true;
         }
-        else if((m_ProcessType == EXTRACT || m_ProcessType == EXTRACT_WITH_HFR) && m_SextractorSolver->sextractionDone())
+        else if((m_ProcessType == EXTRACT || m_ProcessType == EXTRACT_WITH_HFR) && m_ExtractorSolver->extractionDone())
         {
-            m_ExtractorStars = m_SextractorSolver->getStarList();
-            background = m_SextractorSolver->getBackground();
-            m_CalculateHFR = m_SextractorSolver->isCalculatingHFR();
-            if(solverWithWCS)
-                solverWithWCS->appendStarsRAandDEC(m_ExtractorStars);
+            m_ExtractorStars = m_ExtractorSolver->getStarList();
+            background = m_ExtractorSolver->getBackground();
+            m_CalculateHFR = m_ExtractorSolver->isCalculatingHFR();
+            if(hasWCS)
+                wcsData.appendStarsRAandDEC(m_ExtractorStars);
             m_HasExtracted = true;
         }
     }
     else
         m_HasFailed = true;
 
-    if(m_ProcessType != SOLVE || !m_SextractorSolver->hasWCSData() || !loadWCS)
-        m_isRunning = false;
+    m_isRunning = false;
 
     emit ready();
-
-    if(m_ProcessType != SOLVE || !m_SextractorSolver->hasWCSData() || !loadWCS)
-        emit finished();
+    emit finished();
 }
 
-int StellarSolver::whichSolver(SextractorSolver *solver)
+int StellarSolver::whichSolver(ExtractorSolver *solver)
 {
     for(int i = 0; i < parallelSolvers.count(); i++ )
     {
@@ -441,19 +529,18 @@ int StellarSolver::whichSolver(SextractorSolver *solver)
 }
 
 //This slot listens for signals from the child solvers that they are in fact done with the solve
-//If they
 void StellarSolver::finishParallelSolve(int success)
 {
     m_ParallelSolversFinishedCount++;
-    SextractorSolver *reportingSolver = qobject_cast<SextractorSolver*>(sender());
+    ExtractorSolver *reportingSolver = qobject_cast<ExtractorSolver*>(sender());
     if(!reportingSolver)
         return;
 
     if(success == 0 && !m_HasSolved)
     {
-        for(auto solver : parallelSolvers)
+        for(auto &solver : parallelSolvers)
         {
-            disconnect(solver, &SextractorSolver::logOutput, this, &StellarSolver::logOutput);
+            disconnect(solver, &ExtractorSolver::logOutput, this, &StellarSolver::logOutput);
             if(solver != reportingSolver && solver->isRunning())
                 solver->abort();
         }
@@ -465,22 +552,20 @@ void StellarSolver::finishParallelSolve(int success)
 
         numStars = reportingSolver->getNumStarsFound();
         solution = reportingSolver->getSolution();
+        solutionIndexNumber = reportingSolver->getSolutionIndexNumber();
+        solutionHealpix = reportingSolver->getSolutionHealpix();
         m_SolverStars = reportingSolver->getStarList();
 
-        if(reportingSolver->hasWCSData() && loadWCS)
+        if(reportingSolver->hasWCSData())
         {
-            solverWithWCS = reportingSolver;
+            wcsData = reportingSolver->getWCSData();
             hasWCS = true;
-            solverWithWCS->computingWCS = true;
-            disconnect(solverWithWCS, &SextractorSolver::finished, this, &StellarSolver::finishParallelSolve);
-            connect(solverWithWCS, &SextractorSolver::finished, this, &StellarSolver::finishWCS);
-            connect(solverWithWCS, &SextractorSolver::logOutput, this, &StellarSolver::logOutput);
-            solverWithWCS->start();
+            if(m_ExtractorStars.count() > 0)
+                wcsData.appendStarsRAandDEC(m_ExtractorStars);
+            m_isRunning = false;
         }
-        if(m_SextractorType !=
-                EXTRACTOR_BUILTIN) //Note this is just cleaning up the files from the star extraction done prior to the parallel solve.  So for built in, it doesn't even do it, so no files to clean up
-            m_SextractorSolver->cleanupTempFiles();
         m_HasSolved = true;
+        m_ExtractorSolver->cleanupTempFiles();
         emit ready();
     }
     else
@@ -491,48 +576,37 @@ void StellarSolver::finishParallelSolve(int success)
 
     if(m_ParallelSolversFinishedCount == parallelSolvers.count())
     {
-
-        if(m_HasSolved)
-        {
-            //Don't emit finished until WCS is done if we are doing WCS extraction.
-            if(!(loadWCS && hasWCS))
-            {
-                m_isRunning = false;
-                emit finished();
-            }
-        }
-        else
-        {
-            m_isRunning = false;
+        m_isRunning = false;
+        if(!m_HasSolved){
             m_HasFailed = true;
-            emit ready();
-            emit finished();
+            emit ready(); //Since this was emitted earlier if it had solved
         }
+        qDeleteAll(parallelSolvers);
+        parallelSolvers.clear();
+        m_ExtractorSolver->cleanupTempFiles();
+        emit finished();
     }
 }
 
-void StellarSolver::finishWCS()
+QString StellarSolver::raString(double ra)
 {
-    if(solverWithWCS)
-    {
-        wcs_coord = solverWithWCS->getWCSCoord();
-        if(m_ExtractorStars.count() > 0)
-            solverWithWCS->appendStarsRAandDEC(m_ExtractorStars);
-        if(wcs_coord)
-        {
-            hasWCSCoord = true;
-            emit wcsReady();
-        }
-    }
-    m_isRunning = false;
-    emit finished();
+    char rastr[32];
+    ra2hmsstring(ra, rastr);
+    return rastr;
+}
+
+QString StellarSolver::decString(double dec)
+{
+    char decstr[32];
+    dec2dmsstring(dec, decstr);
+    return decstr;
 }
 
 bool StellarSolver::wcsToPixel(const FITSImage::wcs_point &skyPoint, QPointF &pixelPoint)
 {
-    if(hasWCS && solverWithWCS)
+    if(hasWCS)
     {
-        solverWithWCS->wcsToPixel(skyPoint, pixelPoint);
+        wcsData.wcsToPixel(skyPoint, pixelPoint);
         return true;
     }
     return false;
@@ -540,22 +614,25 @@ bool StellarSolver::wcsToPixel(const FITSImage::wcs_point &skyPoint, QPointF &pi
 
 bool StellarSolver::pixelToWCS(const QPointF &pixelPoint, FITSImage::wcs_point &skyPoint)
 {
-    if(hasWCS && solverWithWCS)
+    if(hasWCS)
     {
-        solverWithWCS->pixelToWCS(pixelPoint, skyPoint);
+        wcsData.pixelToWCS(pixelPoint, skyPoint);
         return true;
     }
     return false;
 }
 
-//This is the abort method.  The way that it works is that it creates a file.  Astrometry.net is monitoring for this file's creation in order to abort.
+//This is the abort method.  It works in different ways for the different solvers.
 void StellarSolver::abort()
 {
-    for(auto solver : parallelSolvers)
-        solver->abort();
-    if(m_SextractorSolver)
-        m_SextractorSolver->abort();
-    wasAborted = true;
+  for(auto &solver : parallelSolvers) {
+      solver->abort();
+      solver->wait();
+  }
+  if(m_ExtractorSolver) {
+      m_ExtractorSolver->abort();
+      m_ExtractorSolver->wait();
+  }
 }
 
 //This method checks all the solvers and the internal running boolean to determine if anything is running.
@@ -563,26 +640,95 @@ bool StellarSolver::isRunning() const
 {
     if(parallelSolversAreRunning())
         return true;
-    if(m_SextractorSolver && m_SextractorSolver->isRunning())
+    if(m_ExtractorSolver && m_ExtractorSolver->isRunning())
         return true;
     return m_isRunning;
 }
 
-//This method uses a fwhm value to generate the conv filter the sextractor will use.
-void StellarSolver::createConvFilterFromFWHM(Parameters *params, double fwhm)
+//This method uses a fwhm value to generate the conv filter the star extractor will use.
+QVector<float> StellarSolver::generateConvFilter(SSolver::ConvFilterType filter, double fwhm)
 {
-    params->fwhm = fwhm;
-    params->convFilter.clear();
-    double a = 1;
-    int size = abs(ceil(fwhm * 0.6));
-    for(int y = -size; y <= size; y++ )
+    QVector<float> convFilter;
+    int size = abs(ceil(fwhm));
+    double amplitude = 1.0;
+    if(filter == SSolver::CONV_DEFAULT)
     {
-        for(int x = -size; x <= size; x++ )
+        convFilter = {1, 2, 1,
+                      2, 4, 2,
+                      1, 2, 1
+                     };
+    }
+    else if(filter == SSolver::CONV_CUSTOM)
+    {
+        //Error! Generate Conv Filter should not get called if using a custom conv filter
+    }
+    else if(filter == SSolver::CONV_GAUSSIAN)
+    {
+        for(int y = -size; y <= size; y++ )
         {
-            double value = a * exp( ( -4.0 * log(2.0) * pow(sqrt( pow(x, 2) + pow(y, 2) ), 2) ) / pow(fwhm, 2));
-            params->convFilter.append(value);
+            for(int x = -size; x <= size; x++ )
+            {
+                double value = amplitude * exp( ( -4.0 * log(2.0) * pow(sqrt( pow(x, 2) + pow(y, 2) ), 2) ) / pow(size * 1.5, 2));
+                convFilter.append(value);
+            }
         }
     }
+    else if(filter == SSolver::CONV_MEXICAN_HAT)
+    {
+
+        for(int y = -size; y <= size; y++ )
+        {
+            for(int x = -size; x <= size; x++ )
+            {
+                //Formula inspired by astropy package
+                double rr_ww = (pow(x, 2) + pow(y, 2)) / (2 * pow(size, 2));
+                convFilter.append(amplitude * (1 - rr_ww) * exp(- rr_ww));
+            }
+        }
+    }
+    else if(filter == SSolver::CONV_TOP_HAT)
+    {
+        for(int y = -size; y <= size; y++ )
+        {
+            for(int x = -size; x <= size; x++ )
+            {
+                double a = 0;
+                if(1.2*abs(x)/size + 1.2*abs(y)/size <= 1)
+                    a = 1.0;
+                else a = 0;
+
+                convFilter.append(amplitude * a);
+            }
+        }
+    }
+    else if(filter == SSolver::CONV_RING)
+    {
+        for(int y = -size; y <= size; y++ )
+        {
+            for(int x = -size; x <= size; x++ )
+            {
+                double value1 = amplitude * exp( ( -4.0 * log(2.0) * pow(sqrt( pow(x, 2) + pow(y, 2) ), 2) ) / pow(size, 2));
+                double value2 = amplitude * exp( ( -4.0 * log(2.0) * pow(sqrt( pow(x, 2) + pow(y, 2) ), 2) ) / pow(size/2.0, 2));
+                convFilter.append(value1 - value2);
+            }
+        }
+    }
+    else
+    {
+        convFilter = {1, 2, 1,
+                      2, 4, 2,
+                      1, 2, 1
+                     };
+    }
+    return convFilter;
+}
+
+void StellarSolver::updateConvolutionFilter()
+{
+    if(params.convFilterType == SSolver::CONV_CUSTOM)
+        return;
+    convFilter.clear();
+    convFilter = generateConvFilter(params.convFilterType, params.fwhm);
 }
 
 QList<Parameters> StellarSolver::getBuiltInProfiles()
@@ -594,17 +740,18 @@ QList<Parameters> StellarSolver::getBuiltInProfiles()
     defaultProfile.description = "Default profile. Generic and not optimized for any specific purpose.";
     profileList.append(defaultProfile);
 
-    Parameters fastSolving;
-    fastSolving.listName = "2-SingleThreadSolving";
-    fastSolving.description = "Profile intended for Plate Solving telescopic sized images in a single CPU Thread";
-    fastSolving.multiAlgorithm = NOT_MULTI;
-    fastSolving.minwidth = 0.1;
-    fastSolving.maxwidth = 10;
-    fastSolving.keepNum = 50;
-    fastSolving.initialKeep = 500;
-    fastSolving.maxEllipse = 1.5;
-    createConvFilterFromFWHM(&fastSolving, 4);
-    profileList.append(fastSolving);
+    Parameters singleThreadSolving;
+    singleThreadSolving.listName = "2-SingleThreadSolving";
+    singleThreadSolving.description = "Profile intended for Plate Solving telescopic sized images in a single CPU Thread";
+    singleThreadSolving.multiAlgorithm = NOT_MULTI;
+    singleThreadSolving.minwidth = 0.1;
+    singleThreadSolving.maxwidth = 10;
+    singleThreadSolving.keepNum = 50;
+    singleThreadSolving.initialKeep = 500;
+    singleThreadSolving.maxEllipse = 1.5;
+    singleThreadSolving.convFilterType = SSolver::CONV_GAUSSIAN;
+    singleThreadSolving.fwhm = 4;
+    profileList.append(singleThreadSolving);
 
     Parameters parLargeSolving;
     parLargeSolving.listName = "3-LargeScaleSolving";
@@ -614,33 +761,37 @@ QList<Parameters> StellarSolver::getBuiltInProfiles()
     parLargeSolving.keepNum = 50;
     parLargeSolving.initialKeep = 500;
     parLargeSolving.maxEllipse = 1.5;
-    createConvFilterFromFWHM(&parLargeSolving, 4);
+    parLargeSolving.convFilterType = SSolver::CONV_GAUSSIAN;
+    parLargeSolving.fwhm = 4;
     profileList.append(parLargeSolving);
 
-    Parameters fastSmallSolving;
-    fastSmallSolving.listName = "4-SmallScaleSolving";
-    fastSmallSolving.description = "Profile intended for Plate Solving telescopic sized images";
-    fastSmallSolving.minwidth = 0.1;
-    fastSmallSolving.maxwidth = 10;
-    fastSmallSolving.keepNum = 50;
-    fastSmallSolving.initialKeep = 500;
-    fastSmallSolving.maxEllipse = 1.5;
-    createConvFilterFromFWHM(&fastSmallSolving, 4);
-    profileList.append(fastSmallSolving);
+    Parameters parSmallSolving;
+    parSmallSolving.listName = "4-SmallScaleSolving";
+    parSmallSolving.description = "Profile intended for Plate Solving telescopic sized images";
+    parSmallSolving.minwidth = 0.1;
+    parSmallSolving.maxwidth = 10;
+    parSmallSolving.keepNum = 50;
+    parSmallSolving.initialKeep = 500;
+    parSmallSolving.maxEllipse = 1.5;
+    parSmallSolving.convFilterType = SSolver::CONV_GAUSSIAN;
+    parSmallSolving.fwhm = 4;
+    profileList.append(parSmallSolving);
 
-    Parameters stars;
-    stars.listName = "5-AllStars";
-    stars.description = "Profile for the source extraction of all the stars in an image.";
-    stars.maxEllipse = 1.5;
-    createConvFilterFromFWHM(&stars, 1);
-    stars.r_min = 2;
-    profileList.append(stars);
+    Parameters allStars;
+    allStars.listName = "5-AllStars";
+    allStars.description = "Profile for the source extraction of all the stars in an image.";
+    allStars.maxEllipse = 1.5;
+    allStars.convFilterType = SSolver::CONV_GAUSSIAN;
+    allStars.fwhm = 2;
+    allStars.r_min = 2;
+    profileList.append(allStars);
 
     Parameters smallStars;
     smallStars.listName = "6-SmallSizedStars";
     smallStars.description = "Profile optimized for source extraction of smaller stars.";
     smallStars.maxEllipse = 1.5;
-    createConvFilterFromFWHM(&smallStars, 1);
+    smallStars.convFilterType = SSolver::CONV_GAUSSIAN;
+    smallStars.fwhm = 2;
     smallStars.r_min = 2;
     smallStars.maxSize = 5;
     smallStars.initialKeep = 500;
@@ -652,7 +803,8 @@ QList<Parameters> StellarSolver::getBuiltInProfiles()
     mid.description = "Profile optimized for source extraction of medium sized stars.";
     mid.maxEllipse = 1.5;
     mid.minarea = 20;
-    createConvFilterFromFWHM(&mid, 4);
+    mid.convFilterType = SSolver::CONV_GAUSSIAN;
+    mid.fwhm = 4;
     mid.r_min = 5;
     mid.removeDimmest = 20;
     mid.minSize = 2;
@@ -666,7 +818,8 @@ QList<Parameters> StellarSolver::getBuiltInProfiles()
     big.description = "Profile optimized for source extraction of larger stars.";
     big.maxEllipse = 1.5;
     big.minarea = 40;
-    createConvFilterFromFWHM(&big, 8);
+    big.convFilterType = SSolver::CONV_GAUSSIAN;
+    big.fwhm = 8;
     big.r_min = 20;
     big.minSize = 5;
     big.initialKeep = 500;
@@ -676,21 +829,21 @@ QList<Parameters> StellarSolver::getBuiltInProfiles()
     return profileList;
 }
 
-QList<SSolver::Parameters> StellarSolver::loadSavedOptionsProfiles(QString savedOptionsProfiles)
+QList<SSolver::Parameters> StellarSolver::loadSavedOptionsProfiles(const QString &savedOptionsProfiles)
 {
     QList<SSolver::Parameters> optionsList;
-    if(!QFileInfo(savedOptionsProfiles).exists())
+    if(!QFileInfo::exists(savedOptionsProfiles))
     {
         return StellarSolver::getBuiltInProfiles();
     }
     QSettings settings(savedOptionsProfiles, QSettings::IniFormat);
     QStringList groups = settings.childGroups();
-    foreach(QString group, groups)
+    foreach(const QString &group, groups)
     {
         settings.beginGroup(group);
         QStringList keys = settings.childKeys();
         QMap<QString, QVariant> map;
-        foreach(QString key, keys)
+        foreach(const QString &key, keys)
             map.insert(key, settings.value(key));
         SSolver::Parameters newParams = SSolver::Parameters::convertFromMap(map);
         settings.endGroup();
@@ -781,7 +934,7 @@ void addPathToListIfExists(QStringList *list, QString path)
 {
     if(list)
     {
-        if(QFileInfo(path).exists())
+        if(QFileInfo::exists(path))
             list->append(path);
     }
 }
@@ -807,20 +960,10 @@ QStringList StellarSolver::getDefaultIndexFolderPaths()
     return indexFilePaths;
 }
 
-
-
-FITSImage::wcs_point * StellarSolver::getWCSCoord()
-{
-    if(hasWCS && hasWCSCoord)
-        return wcs_coord;
-    else
-        return nullptr;
-}
-
 bool StellarSolver::appendStarsRAandDEC(QList<FITSImage::Star> &stars)
 {
-    if(solverWithWCS)
-        return solverWithWCS->appendStarsRAandDEC(stars);
+    if(hasWCS)
+        return wcsData.appendStarsRAandDEC(stars);
     return false;
 }
 
@@ -868,18 +1011,18 @@ bool StellarSolver::getAvailableRAM(double &availableRAM, double &totalRAM)
 }
 
 //This should determine if enough RAM is available to load all the index files in parallel
-bool StellarSolver::enoughRAMisAvailableFor(QStringList indexFolders)
+bool StellarSolver::enoughRAMisAvailableFor(const QStringList &indexFolders)
 {
     double totalSize = 0;
 
-    foreach(QString folder, indexFolders)
+    foreach(const QString &folder, indexFolders)
     {
         QDir dir(folder);
         if(dir.exists())
         {
             dir.setNameFilters(QStringList() << "*.fits" << "*.fit");
             QFileInfoList indexInfoList = dir.entryInfoList();
-            foreach(QFileInfo indexInfo, indexInfoList)
+            foreach(const QFileInfo &indexInfo, indexInfoList)
                 totalSize += indexInfo.size();
         }
 
